import { useRef, useEffect, useState, useCallback, createElement, Fragment, cloneElement, Children } from 'react';
import { Portal } from 'reakit/Portal';

function ownKeys(object, enumerableOnly) {
  var keys = Object.keys(object);

  if (Object.getOwnPropertySymbols) {
    var symbols = Object.getOwnPropertySymbols(object);

    if (enumerableOnly) {
      symbols = symbols.filter(function (sym) {
        return Object.getOwnPropertyDescriptor(object, sym).enumerable;
      });
    }

    keys.push.apply(keys, symbols);
  }

  return keys;
}

function _objectSpread2(target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i] != null ? arguments[i] : {};

    if (i % 2) {
      ownKeys(Object(source), true).forEach(function (key) {
        _defineProperty(target, key, source[key]);
      });
    } else if (Object.getOwnPropertyDescriptors) {
      Object.defineProperties(target, Object.getOwnPropertyDescriptors(source));
    } else {
      ownKeys(Object(source)).forEach(function (key) {
        Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key));
      });
    }
  }

  return target;
}

function _defineProperty(obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
}

function useAnimationFrame(update) {
  var pause = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;
  var deps = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : [];
  var latestAnimationFrame = useRef(null);
  useEffect(() => {
    if (typeof pause === "function" ? pause() : pause) {
      return;
    }

    latestAnimationFrame.current = requestAnimationFrame(() => {
      update();
    });
    return () => {
      if (latestAnimationFrame.current) {
        cancelAnimationFrame(latestAnimationFrame.current);
      }
    }; // eslint-disable-next-line react-hooks/exhaustive-deps
  }, [update, pause, ...deps]);
}

var fitsOnBottom = viewportOffset => (elemRect, viewportRect) => elemRect.bottom < viewportRect.height - viewportOffset;
var fitsOnRight = viewportOffset => (elemRect, viewportRect) => elemRect.right < viewportRect.width - viewportOffset;
var getFitOffsets = viewportOffset => elemRect => {
  var viewportRect = {
    width: window.innerWidth,
    height: window.innerHeight
  };
  var offsets = {
    x: 0,
    y: 0
  };

  if (!fitsOnRight(viewportOffset)(elemRect, viewportRect)) {
    offsets.x = viewportRect.width - viewportOffset - elemRect.right;
  }

  if (!fitsOnBottom(viewportOffset)(elemRect, viewportRect)) {
    offsets.y = viewportRect.height - viewportOffset - elemRect.bottom;
  }

  return offsets;
};

var DEFAULT_POSITION = {
  top: 0,
  left: 0
};
function useFitInViewport(elemRef, targetPos) {
  var viewportOffset = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 0;
  var [style, setStyle] = useState(targetPos);
  useEffect(() => {
    setStyle(targetPos);
  }, [targetPos]);
  useAnimationFrame(() => {
    if (!elemRef.current) {
      return;
    }

    var {
      x,
      y
    } = getFitOffsets(viewportOffset)(elemRef.current.getBoundingClientRect());
    setStyle(prevStyle => {
      var nextStyle = {
        top: (prevStyle || DEFAULT_POSITION).top + y,
        left: (prevStyle || DEFAULT_POSITION).left + x
      };
      return nextStyle;
    });
  }, () => !elemRef.current, [style]);
  return {
    style
  };
}

var setEventListeners = (events, fn, elem, set) => {
  events.forEach(event => {
    (set === "add" ? elem.addEventListener : elem.removeEventListener)(event, fn, {
      capture: true,
      passive: true
    });
  });
};

function useAttachEventListeners(elem, events, cb) {
  useEffect(() => {
    if (!elem) {
      return;
    }

    setEventListeners(events, cb, elem, "add");
    return () => {
      setEventListeners(events, cb, elem, "remove");
    };
  }, [cb, elem, events]);
}

var GET_POSITION_DEFAULT = e => ({
  top: e.pageY,
  left: e.pageX
});

var HIDE_ON_DEFAULT = "resize contextmenu mousedown click scroll keydown";

var SHOULD_OPEN_DEFAULT = () => true;

function MenuWindow(_ref) {
  var {
    children,
    getPosition = GET_POSITION_DEFAULT,
    hideOn = HIDE_ON_DEFAULT,
    render,
    shouldOpen = SHOULD_OPEN_DEFAULT,
    onOpen,
    onClose,
    viewportOffset = 8
  } = _ref;
  var latestEvent = useRef(null);
  var [pos, setPos] = useState(null);
  var contentRef = useRef(null);
  var openWindow = useCallback(e => {
    onOpen && onOpen(e);
    setPos(getPosition(e));
  }, [onOpen, setPos, getPosition]);
  var closeWindow = useCallback(e => {
    setPos(null);
    onClose && onClose(e);
  }, [setPos, onClose]);
  var handleContextMenu = useCallback(e => {
    e.preventDefault();
    e.stopPropagation();
    e.persist();
    latestEvent.current = e;

    if (shouldOpen(e)) {
      openWindow(e);
    }
  }, [shouldOpen, openWindow]);
  var handleClose = useCallback(e => {
    /**
     * Ignore all hideOn events if they're triggered on the content element or any of its children.
     */
    if (!contentRef.current || e.target !== contentRef.current && !contentRef.current.contains(e.target)) {
      closeWindow(e);
    }
  }, [contentRef, closeWindow]);
  useAttachEventListeners(pos ? window : null, hideOn.split(" "), handleClose);
  var renderedContent = latestEvent.current && pos ? render(latestEvent.current, {
    close: () => setPos(null)
  }) : null;
  var {
    style: contentStyles
  } = useFitInViewport(contentRef, pos, viewportOffset);
  return createElement(Fragment, null, cloneElement(Children.only(children), {
    onContextMenu: handleContextMenu
  }), renderedContent ? createElement(Portal, null, cloneElement(renderedContent, {
    ref: contentRef,
    style: _objectSpread2(_objectSpread2({}, renderedContent.props.style), {}, {
      position: "absolute"
    }, contentStyles)
  })) : null);
}

export { MenuWindow };
